@@ -16,6 +16,8 @@
 import numpy
 import gvar
 import lsqfit
+import time
+import collections
 from ._utilities import multiminex, gammaQ
 
 
@@ -65,32 +67,47 @@ def empbayes_fit(z0, fitargs, **minargs):
 ##
     
 def wavg(xa, svdcut=None, svdnum=None, rescale=True, covfac=None):
-    """ Weighted average of 1d-sequence of |GVar|\s or arrays of |GVar|\s.
+    """ Weighted average of |GVar|\s or arrays/dicts of |GVar|\s.
         
-    The weighted average of several |GVar|\s is what one obtains from a 
-    least-squares fit of the collection of |GVar|\s to the one-parameter
-    fit function ``def f(p): return p[0]``. The average is the best-fit
-    value for ``p[0]``. |GVar|\s with smaller standard deviations carry
-    more weight than those with larger standard deviations. The averages
-    computed by ``wavg`` take account of correlations between the |GVar|\s.
+    The weighted average of several |GVar|\s is what one obtains from
+    a  least-squares fit of the collection of |GVar|\s to the one-
+    parameter fit function ``def f(p): return N * [p[0]]`` where ``N``
+    is the number of |GVar|\s. The average is the best-fit value for
+    ``p[0]``.  |GVar|\s with smaller standard deviations carry more
+    weight than those with larger standard deviations. The averages
+    computed by ``wavg`` take account of correlations between the
+    |GVar|\s.
         
     Typical usage is::
         
         x1 = gvar.gvar(...)
         x2 = gvar.gvar(...)
-        xavg = wavg([x1, x2])   # weighted average of x1 and x2
-        
-    In this example, ``x1`` and ``x2`` could be replaced by arrays of 
-    |GVar|\s, in which case ``xavg`` is an array as well: for example, ::
-        
+        x3 = gvar.gvar(...)
+        xavg = wavg([x1, x2, x3])   # weighted average of x1, x2 and x3
+    
+    The individual |GVar|\s in this example can be  replaced by
+    multidimensional distributions, represented by arrays of |GVar|\s
+    or dictionaries of |GVar|\s or arrays of |GVar|\s. For example, ::
+
         x1 = [gvar.gvar(...), gvar.gvar(...)]
         x2 = [gvar.gvar(...), gvar.gvar(...)]
-        xavg = wavg([x1, x2])   # xavg[i] is wgtd avg of x1[i] and x2[i]
-        
+        x3 = [gvar.gvar(...), gvar.gvar(...)]
+        xavg = wavg([x1, x2, x3])   
+            # xavg[i] is wgtd avg of x1[i], x2[i], x3[i]
+
+    and ::
+
+        x1 = dict(a=[gvar.gvar(...), gvar.gvar(...)], b=gvar.gvar(...))
+        x2 = dict(a=[gvar.gvar(...), gvar.gvar(...)], b=gvar.gvar(...))
+        x3 = dict(a=[gvar.gvar(...), gvar.gvar(...)], b=gvar.gvar(...))
+        xavg = wavg([x1, x2, x3])   
+            # xavg['a'][i] is wgtd avg of x1['a'][i], x2['a'][i], x3['a'][i]
+            # xavg['b'] is gtd avg of x1['b'], x2['b'], x3['b']       
         
     :param xa: The |GVar|\s to be averaged. ``xa`` is a one-dimensional
-        sequence of |GVar|\s or of arrays of |GVar|\s, all of the same
-        shape.
+        sequence of |GVar|\s, or of arrays of |GVar|\s, or of dictionaries 
+        containing |GVar|\s or arrays of |GVar|\s. All ``xa[i]`` must
+        have the same shape.
     :param svdcut: If positive, eigenvalues of the ``xa`` covariance matrix
         that are smaller than ``svdcut`` times the maximum eigenvalue 
         are replaced by ``svdcut`` times the maximum eigenvalue. If 
@@ -127,58 +144,97 @@ def wavg(xa, svdcut=None, svdnum=None, rescale=True, covfac=None):
     .. attribute:: wavg.Q
         
         Quality factor `Q` for fit.
+
+    .. attribute:: wavg.time
+
+        Time required to do average.
         
     """
+    cpu_time = time.clock()
     xa = numpy.asarray(xa)
+    if xa.size == 0:
+        return None
     s = None
     svdcorrection = []
-    try:
+    if len(xa.shape) > 1:
         ## xa is an array of arrays ##
         shape = xa[0].shape
         xaflat = [xai.flat for xai in xa]
-        ans = [wavg(xtuple) for xtuple in zip(*xaflat)]
+        chi2 = 0.0
+        dof = 0
+        ans = []
+        for xtuple in zip(*xaflat):
+            ans.append(wavg(
+                xtuple, 
+                svdcut=svdcut, svdnum=svdnum,
+                rescale=rescale, covfac=covfac
+                ))
+            chi2 += wavg.chi2
+            dof += wavg.dof
+            svdcorrection.append(wavg.svdcorrection)
         ans = numpy.array(ans)
         ans.shape = shape
-        return ans
-        ##
-    except AttributeError:
-        pass
-    cov = gvar.evalcov(xa)
-    if covfac is not None:
-        cov *= covfac
-    ## invert cov ## 
-    if numpy.all(numpy.diag(numpy.diag(cov))==cov):
-        ## cov is diagonal ## 
-        invcov = 1./numpy.diag(cov)
-        dof = len(xa)-1
-        ans = numpy.dot(invcov, xa)/sum(invcov)
-        chi2 = numpy.sum((xa-ans)**2*invcov).mean
-        ##
+    elif hasattr(xa[0], 'keys'):
+        data = collections.OrderedDict()
+        # collect copies of data -- different dicts can have different keys
+        for xai in xa:
+            for k in xai:
+                if k in data:
+                    data[k].append(xai[k])
+                else:
+                    data[k] = [xai[k]]
+        ans = gvar.BufferDict()
+        chi2 = 0.0
+        dof = 0
+        for k in data:
+            ans[k] = wavg(
+                data[k], 
+                svdcut=svdcut, svdnum=svdnum,
+                rescale=rescale, covfac=covfac
+                )
+            chi2 += wavg.chi2
+            dof += wavg.dof
+            svdcorrection.append(wavg.svdcorrection)
     else:
-        ## cov is not diagonal ##
-        if (svdcut is None or svdcut==0) and (svdnum is None or svdnum<0):
-            invcov = numpy.linalg.inv(cov)
+        cov = gvar.evalcov(xa)
+        if covfac is not None:
+            cov *= covfac
+        ## invert cov ## 
+        if numpy.all(numpy.diag(numpy.diag(cov))==cov):
+            ## cov is diagonal ## 
+            invcov = 1./numpy.diag(cov)
             dof = len(xa)-1
-        else:
-            ## apply svdcuts; compute conditioned inverse ## 
-            s = gvar.SVD(cov, svdcut=svdcut, svdnum=svdnum, rescale=rescale,
-                         compute_delta=True)
-            invcov = numpy.sum(numpy.outer(wj, wj) for wj 
-                                    in reversed(s.decomp(-1)))
-            dof = len(s.val)-1
-            if s.delta is not None:
-                svdcorrection = sum(s.delta)
+            ans = numpy.dot(invcov, xa)/sum(invcov)
+            chi2 = numpy.sum((xa-ans)**2*invcov).mean
             ##
+        else:
+            ## cov is not diagonal ##
+            if (svdcut is None or svdcut==0) and (svdnum is None or svdnum<0):
+                invcov = numpy.linalg.inv(cov)
+                dof = len(xa)-1
+            else:
+                ## apply svdcuts; compute conditioned inverse ## 
+                s = gvar.SVD(cov, svdcut=svdcut, svdnum=svdnum, rescale=rescale,
+                             compute_delta=True)
+                invcov = numpy.sum(numpy.outer(wj, wj) for wj 
+                                        in reversed(s.decomp(-1)))
+                dof = len(s.val)-1
+                if s.delta is not None:
+                    svdcorrection = sum(s.delta)
+                ##
+            ##
+            sum_invcov = numpy.sum(invcov, axis=1)
+            ans = numpy.dot(sum_invcov, xa)/sum(sum_invcov)
+            chi2 = numpy.dot((xa-ans), numpy.dot(invcov, (xa-ans))).mean
         ##
-        sum_invcov = numpy.sum(invcov, axis=1)
-        ans = numpy.dot(sum_invcov, xa)/sum(sum_invcov)
-        chi2 = numpy.dot((xa-ans), numpy.dot(invcov, (xa-ans))).mean
-    ##
     wavg.chi2 = chi2 
     wavg.dof = dof
     wavg.Q = gammaQ(dof/2., chi2/2.)
     wavg.s = s
     wavg.svdcorrection = svdcorrection
+    wavg.time = time.clock() - cpu_time
     return ans
 ##
 
+if __name__ == '__main__':
+    pass
